#include "graphicalnode.h"
#include "scenetree.h"
#include "glm/ext.hpp"

void GraphicalNode::setSceneTree(SceneTree *tree){
    SceneNode::setSceneTree(tree);
    if(tree)
        tree->addToIndex(this);
}

void GraphicalNode::toggleVisibility(){
    m_visible = !m_visible;
    if(m_visible){
        m_scene->addToIndex(this);
    }else{
        m_scene->removeFromIndex(this);
    }
}

// tools
void GraphicalNode::moveTo(const glm::vec3 &position)
{
    m_transform = glm::translate(glm::mat4(), position);
    m_transformChanged = true;
}

void GraphicalNode::translate(const glm::vec3 &vector)
{
    m_transform = glm::translate(m_transform, vector);
    m_transformChanged = true;
}

void GraphicalNode::lookAt(const glm::vec3 &target, const glm::vec3 &upVector)
{
    glm::vec3 pos = glm::vec3(m_transform[3]);
    m_transform = glm::lookAt(pos, target, upVector);
    m_transformChanged = true;
}

void GraphicalNode::rotate(float angle, const glm::vec3 &vector)
{
    m_transform = glm::rotate(m_transform, angle, vector);
    m_transformChanged = true;
}

void GraphicalNode::scale(const glm::vec3 &scaleFactor)
{
    m_transform = glm::scale(m_transform, scaleFactor);
    m_transformChanged = true;
}

// setters
void GraphicalNode::setTransform(const glm::mat4 &transform)
{
    m_transform = transform;
    m_transformChanged = true;
}

// could be optimised by just storing a pointer, but unexpected behavior may happen ?
void GraphicalNode::setParentTransform(const glm::mat4 &transform)
{
    m_parentTransform = transform;
    m_transformChanged = true;
}
