#ifndef GRAPHICALNODE_H
#define GRAPHICALNODE_H

#include "scenenode.h"
#include <glm/mat4x4.hpp>
#include <glm/vec3.hpp>

class SceneTree;

class GraphicalNode : public SceneNode
{
protected:
    // m_parentTransform is the base transform for this element
    glm::mat4 m_parentTransform;
    // m_transform is the relative transformation matrix of this node
    glm::mat4 m_transform;

    bool m_visible;

    bool m_transformChanged;

public:
    GraphicalNode() : m_visible(true), m_transformChanged(true) {}

    void toggleVisibility();
    virtual void setSceneTree(SceneTree* tree);

    void setTransform(const glm::mat4 &transform);

    void setParentTransform(const glm::mat4 &transform);

    const glm::mat4& getTransform() { return m_transform; }

    // transformation tools :
    void moveTo(const glm::vec3 &position);
    void translate(const glm::vec3 &vector);
    void lookAt(const glm::vec3 &target, const glm::vec3 &upVector = glm::vec3(0, 1, 0));
    void rotate(float angle, const glm::vec3 &vector);
    void scale(const glm::vec3 &scaleFactor);
};

#endif // GRAPHICALNODE_H
